/*
A D-Bus client for print_svc
*/
#define DBUS_API_SUBJECT_TO_CHANGE
#include <stdio.h>
#include <unistd.h>
#include <dbus/dbus.h>
#include <dbus/dbus-protocol.h>

#define SAMPLE_SIGNAL	"SAMPLE_SINGAL"

const char* my_path = "/org/mahir/greenAp";
const char* path = "/org/mahir/greenAp/print_svc1";
const char* svc_name= "org.mahir.greenAp.print_svc1";
const char* my_inf= "org.mahir.greenAp";

DBusMessage* send_signal(DBusConnection* conn, DBusMessage* msg)
{
	msg = dbus_message_new_signal(my_path, my_inf, SAMPLE_SIGNAL);
	if(msg == NULL)
	{
		fprintf(stderr, "Creation of New Signal failed, LINE: %d\n", 
					__LINE__);
		exit(1);
	}
#if 1
	//Passing a string with dot[s] makes the daemon crash
	//Usage is same as in dbus-message.c:6894
	if(!dbus_message_set_sender(msg, "org.mahir.greenAp.abc"))
		printf(stderr, "send_signal: No mem at LINE: %d\n", __LINE__);

	if(!dbus_message_append_args(msg, DBUS_TYPE_STRING, "Hello.string", 
					DBUS_TYPE_INVALID))
		printf(stderr, "send_signal: No mem at LINE: %d\n", __LINE__);

#endif
	return(dbus_message_copy(msg));
}

int main(int argc, char* argv[])
{
	DBusConnection* conn;
	DBusError error;
	DBusMessage* msg;
	int ret, succ;
        char buf;

	dbus_error_init(&error);

	conn = dbus_bus_get(DBUS_BUS_SYSTEM, &error);
	if (conn == NULL) {
		fprintf(stderr, "Error when connecting to the bus: %s,
					 LINE: %d\n", error.message, __LINE__);
		exit(1);
	}

	/* Check for the existance of the service */
	ret = dbus_bus_service_exists(conn, svc_name, &error);
	if (ret)
		printf("service %s exists\n", svc_name);
	else
		printf("service %s does not exist\n", svc_name);

	/* activate the service */
	succ = dbus_bus_activate_service(conn, svc_name, 0,
				 	/* flags not used (accord. to spec) */
					&ret, &error);
	if (succ)
		fprintf(stderr, "activation succeeded\n");
	else
		fprintf(stderr, "activation failed\n");

	if (ret == DBUS_ACTIVATION_REPLY_ACTIVATED)
		fprintf(stderr, "Service Activated\n");
	else if (ret == DBUS_ACTIVATION_REPLY_ALREADY_ACTIVE)
		fprintf(stderr, "Service was already active\n");
	else
		fprintf(stderr, "unknown return value!\n");

	printf("Sening a SIGNAL to server\n");
	msg = send_signal(conn, msg);

	int countLoop = 0;
	while(countLoop++ < 30)
	{
		printf("Sending for %dth time\n", countLoop);
		countLoop++;
		succ = dbus_connection_send(conn, msg, &ret);
		if (!succ) {
			printf("sending of the message failed\n");
			exit(1);
		} else
			printf("the message was put into the queue\n");
	
		DBusDispatchStatus dStatus = dbus_connection_get_dispatch_status(conn);
		if(dStatus == DBUS_DISPATCH_DATA_REMAINS)
			printf("Data yet to be dispatched\n");
		if(dStatus == DBUS_DISPATCH_COMPLETE)
			printf("Data dispatched\n");
		if(dStatus == DBUS_DISPATCH_NEED_MEMORY)
			printf("OOM for dispatch\n");
	}
	dbus_connection_flush(conn);
	dbus_message_unref(msg);
	
	return 1;
}
