/*
A D-Bus service that prints a string arg. to the stdout
*/
#define DBUS_API_SUBJECT_TO_CHANGE
#include <stdio.h>
#include <signal.h>
#include <dbus/dbus.h>
#include <glib/dbus-glib.h>

DBusHandlerResult msg_handler1(DBusConnection* c,
				 DBusMessage* msg,
				 void* user_data);

const char* my_svc = "org.mahir.greenAp.print_svc1";

static dbus_int32_t connection_slot = -1;
const char* path[] = {"org", "mahir", "greenAp", "print_svc1", NULL};

dbus_uint32_t send_error;

DBusObjectPathVTable vtable = {
	.message_function = msg_handler1,
	.unregister_function = NULL
};

static void signal_handler(int sig)
{
	switch(sig)
	{
	case SIGPIPE:
		printf("Got SIGPIPE\n");
		break;
	case SIGINT:
		printf("Got Ctrl C exiting...\n");
		exit(0);
		break;
	default:
		return;
	}
}

DBusHandlerResult msg_handler1(DBusConnection* c,
DBusMessage* msg, void* user_data)
{
	int ret;
	char* p;
	DBusMessageIter iter;
	struct DBusConnection* conn = (struct DBusConnection*)user_data;

	dbus_message_iter_init(msg, &iter);

	/* Check whether the connection is connected */
	if(dbus_connection_get_is_connected(conn))
		printf("Connection is still connected\n");
	else
		printf("Connection is no more connected\n");

	/* Check the message type */
	dbus_uint32_t type = dbus_message_get_type(msg);
	switch(type)
	{
	case DBUS_MESSAGE_TYPE_METHOD_CALL:
		printf("Message received is a Method Call\n");
		break;
	case DBUS_MESSAGE_TYPE_SIGNAL:
		printf("Message received is a SIGNAL\n");
		break;
	}

	/* check the target of the message */
	const char* dest = dbus_message_get_destination(msg);
	if (strcmp(dest, my_svc) == 0)
		fprintf(stderr, "addressed to me\n");
	else
		fprintf(stderr, "addressed to %s\n", dest);

	/* Path related APIs */
	const char* obj_path = dbus_message_get_path(msg);
	fprintf(stderr, "Object Path of the Message is %s\n", obj_path);

	char** dPath = NULL;
	int j = 0;
	dbus_message_get_path_decomposed(msg, &dPath);
	printf("The DECOMPOSED path of the message is : ");
	while(dPath[j] != NULL)
		printf("\n %s", dPath[j++]);
	printf("\n");

	/* Interface */
	const char* obj_inf = dbus_message_get_interface(msg);
	fprintf(stderr, "Interface of the Message is %s\n", obj_inf);

	/* Member */
	const char* obj_method = dbus_message_get_member(msg);
	fprintf(stderr, "Method of the Message is %s\n", obj_method);

	/* Read the contents of the message */
	p = dbus_message_iter_get_string(&iter);
	fprintf(stderr, "the string argument: %s\n", p);

	printf("msg_handler1: Returning from the fallback...\n");
	return DBUS_HANDLER_RESULT_HANDLED;
}

int main(int argc, char* argv[])
{
	struct DBusConnection* conn=NULL;
	DBusError error_src;
	DBusError error1,error;

	/* DBusHandleMessageFunction* handler; */
	DBusMessage* msg;
	GMainLoop* loop;
	int ret;
	char* mtable[2];

	signal(SIGPIPE, signal_handler);
	signal(SIGINT, signal_handler);

	loop = g_main_loop_new(NULL, FALSE);

	/* make a connection to the users login session bus
	(we should use DBUS_BUS_ACTIVATION when were executed by D-Bus) */
	dbus_error_init(&error_src);
	dbus_error_init(&error1);
	dbus_error_init(&error);

	conn = dbus_bus_get(DBUS_BUS_SYSTEM, &error); 
	if (conn == NULL) {
		fprintf(stderr, "Error when connecting to the bus: %s\n", error.message);
		dbus_error_free(&error);
		exit(1);
	}

	dbus_connection_set_change_sigpipe(0);

	/* Checking if the error is set */
	if(dbus_error_is_set(&error))
		printf("dbus_bus_get returned error\n");
	else
		printf("dbus_bus_get successful \n");

while(ret != DBUS_SERVICE_REPLY_PRIMARY_OWNER)
{
	/* tell D-Bus about our service */
	ret = dbus_bus_acquire_service(conn, my_svc, 
		0,
		/* options in the spec seem outdated */ &error);

	if (ret != DBUS_SERVICE_REPLY_PRIMARY_OWNER) {
		/* we want it now or never */
		fprintf(stderr, "Couldnt immediately acquire the service\n");
		fprintf(stderr, "Ret is %d\n", ret);
		fprintf(stderr, "Error is %s\n", error.message);
		dbus_error_init(&error);
		sleep(1);
		continue;
	}
	else break;
}

	/* setup D-Bus to use the Glib main loop */
	dbus_connection_setup_with_g_main(conn, NULL);
	if (!dbus_connection_register_object_path(conn, path, &vtable, (void*)conn))
	{
		fprintf(stderr, "couldnt register the object path\n");
		dbus_error_free(&error);
		dbus_connection_free_data_slot(&connection_slot);
		dbus_connection_unref(conn);
		exit(1);
	}

	char** children = NULL;
	/* Ensuring that the fallback has been registered */
	dbus_bool_t bFlag = dbus_connection_list_registered(conn, path, &children);
	printf("Val returned by dbus_connection_list_registered is:%d\n", bFlag);
	int i = 0;
	while(children[i])
	{
		printf("ChildNode Name is %s\n", children[i]);
		i++;
	}

	printf("Going to the main loop...\n");
	g_main_loop_run(loop);

	dbus_error_free(&error);
	dbus_free_string_array(children);
	dbus_connection_free_data_slot(&connection_slot);
	dbus_connection_unref(conn);
	printf("Going to call exit()...\n");
	exit(0);
}
